// Generated by CoffeeScript 2.3.2
(function() {
  var DEFAULT_CONFIG, _, async, build, config, createIt, expandUrl, linkFilter, linkFinder, localItFunction, setIt, templates, validate;

  _ = require('lodash');

  async = require('async');

  templates = require('uri-templates');

  linkFinder = require('./link_finder');

  linkFilter = require('./link_filter');

  build = require('./request_builder');

  DEFAULT_CONFIG = function() {
    return {
      url: null,
      options: {},
      templateValues: {},
      samplePercentage: 100,
      getLinks: linkFinder.getLinks,
      validate: function() {
        return true;
      },
      recover: function() {
        return false;
      }
    };
  };

  localItFunction = null;

  config = DEFAULT_CONFIG();

  exports.getLinks = function(res) {
    return linkFilter.filter(config.getLinks(res.body), config.samplePercentage);
  };

  exports.setConfig = function(userconfig) {
    return config = _.extend({}, DEFAULT_CONFIG(), userconfig);
  };

  setIt = function(it) {
    return localItFunction = it;
  };

  createIt = (url, templateValues) => {
    return localItFunction(url, (done) => {
      return build.request(url, config.options).end((res) => {
        return exports.processResponse(url, res, templateValues, done);
      });
    });
  };

  exports.createItWithResult = function(url, err) {
    return localItFunction(url, function(done) {
      return done(err);
    });
  };

  exports.processResponse = (parent, res, templateValues, done) => {
    var err;
    if (!res.ok) {
      if (!config.recover(res)) {
        err = `Bad status ${res.status} for url ${res.url}`;
      }
      return done(err);
    } else {
      try {
        if (!validate(parent, res)) {
          return done(`Not a valid response: ${res.body}`);
        }
      } catch (error) {
        err = error;
        return done(err);
      }
    }
    return describe(`${parent}`, function() {
      var requests;
      requests = _.map(exports.getLinks(res), function(link) {
        return function(callback) {
          var expandedLink;
          linkFilter.processLink(link);
          expandedLink = expandUrl(link, templateValues);
          return build.request(expandedLink, config.options).end((res) => {
            return exports.processResponse(expandedLink, res, templateValues, function(err) {
              return callback(null, {
                err,
                link: expandedLink
              });
            });
          });
        };
      });
      return async.parallel(requests, function(err, results) {
        results.forEach(function(result) {
          return exports.createItWithResult(result.link, result.err);
        });
        return done();
      });
    });
  };

  expandUrl = function(url, values) {
    if (_.isObject(values) && !_.isEmpty(values)) {
      return templates(url).fillFromObject(values);
    } else {
      return url;
    }
  };

  validate = function(url, res) {
    return config.validate(url, res.body);
  };

  exports.startCrawl = function(config, it) {
    var expandedUrl;
    exports.reset();
    if (it) {
      setIt(it);
    }
    exports.setConfig(config);
    expandedUrl = expandUrl(config.url, config.templateValues);
    linkFilter.processLink(expandedUrl);
    return createIt(expandedUrl, config.templateValues);
  };

  exports.reset = function() {
    linkFilter.reset();
    localItFunction = null;
    return config = DEFAULT_CONFIG();
  };

}).call(this);
